@@ -40,8 +40,9 @@
 #include "err.h"
 
 
-/* Convenient macro for closing files */
+/* Convenient macros for closing files */
 #define FCLOSE(fp) (((fp)&&((fp)!=stdin)) ? (fclose(fp),(fp)=NULL) : ((fp)=NULL))
+#define PCLOSE(fp) (((fp)&&((fp)!=stdin)) ? (pclose(fp),(fp)=NULL) : ((fp)=NULL))
 
 /* Define the structures needed by the file caching system */
 typedef struct cache {
@@ -91,12 +92,12 @@ static FILE *fp;
 static IncludeStruct IStack[INCLUDE_NEST];
 static int IStackPtr = 0;
 
-static int ReadLineFromFile (void);
-static int CacheFile (char const *fname);
+static int ReadLineFromFile (int use_pclose);
+static int CacheFile (char const *fname, int use_pclose);
 static void DestroyCache (CachedFile *cf);
 static int CheckSafety (void);
 static int PopFile (void);
-
+static int IncludeCmd(char const *);
 static void OpenPurgeFile(char const *fname, char const *mode)
 {
     DynamicBuffer fname_buf;
@@ -167,7 +168,7 @@ int ReadLine(void)
     }
 
 /* Not cached.  Read from the file. */
-    return ReadLineFromFile();
+    return ReadLineFromFile(0);
 }
 
 /***************************************************************/
@@ -177,7 +178,7 @@ int ReadLine(void)
 /*  Read a line from the file pointed to by fp.                */
 /*                                                             */
 /***************************************************************/
-static int ReadLineFromFile(void)
+static int ReadLineFromFile(int use_pclose)
 {
     int l;
     char copy_buffer[4096];
@@ -200,7 +201,11 @@ static int ReadLineFromFile(void)
 	    return E_IO_ERR;
 	}
 	if (feof(fp)) {
-	    FCLOSE(fp);
+            if (use_pclose) {
+                PCLOSE(fp);
+            } else {
+                FCLOSE(fp);
+            }
 	    if ((DBufLen(&buf) == 0) &&
 		(DBufLen(&LineBuffer) == 0) && PurgeMode) {
 		if (PurgeFP != NULL && PurgeFP != stdout) fclose(PurgeFP);
@@ -248,7 +253,11 @@ static int ReadLineFromFile(void)
 		if (PurgeFP != stdout) fclose(PurgeFP);
 		PurgeFP = NULL;
 	    }
-	    FCLOSE(fp);
+            if (use_pclose) {
+                PCLOSE(fp);
+            } else {
+                FCLOSE(fp);
+            }
 	    DBufFree(&LineBuffer);
 	    CurLine = DBufValue(&LineBuffer);
 	}
@@ -282,9 +291,6 @@ int OpenFile(char const *fname)
 	PurgeFP = NULL;
     }
 
-/* Assume we own the file for now */
-    RunDisabled &= ~RUN_NOTOWNER;
-
 /* If it's in the cache, get it from there. */
 
     while (h) {
@@ -297,7 +303,9 @@ int OpenFile(char const *fname)
 	    LineNo = 0;
 	    if (!h->ownedByMe) {
 		RunDisabled |= RUN_NOTOWNER;
-	    }
+	    } else {
+		RunDisabled &= ~RUN_NOTOWNER;
+            }
 	    if (FileName) return OK; else return E_NO_MEM;
 	}
 	h = h->next;
@@ -306,6 +314,7 @@ int OpenFile(char const *fname)
 /* If it's a dash, then it's stdin */
     if (!strcmp(fname, "-")) {
 	fp = stdin;
+        RunDisabled &= ~RUN_NOTOWNER;
 	if (PurgeMode) {
 	    PurgeFP = stdout;
 	}
@@ -325,7 +334,7 @@ int OpenFile(char const *fname)
     CLine = NULL;
     if (ShouldCache) {
 	LineNo = 0;
-	r = CacheFile(fname);
+	r = CacheFile(fname, 0);
 	if (r == OK) {
 	    fp = NULL;
 	    CLine = CachedFiles->cache;
@@ -353,7 +362,7 @@ int OpenFile(char const *fname)
 /*  Returns an indication of success or failure.               */
 /*                                                             */
 /***************************************************************/
-static int CacheFile(char const *fname)
+static int CacheFile(char const *fname, int use_pclose)
 {
     int r;
     CachedFile *cf;
@@ -368,14 +377,22 @@ static int CacheFile(char const *fname)
     cf = NEW(CachedFile);
     if (!cf) {
 	ShouldCache = 0;
-	FCLOSE(fp);
+        if (use_pclose) {
+            PCLOSE(fp);
+        } else {
+            FCLOSE(fp);
+        }
 	return E_NO_MEM;
     }
     cf->cache = NULL;
     cf->filename = StrDup(fname);
     if (!cf->filename) {
 	ShouldCache = 0;
-	FCLOSE(fp);
+        if (use_pclose) {
+            PCLOSE(fp);
+        } else {
+            FCLOSE(fp);
+        }
 	free(cf);
 	return E_NO_MEM;
     }
@@ -388,11 +405,15 @@ static int CacheFile(char const *fname)
 
 /* Read the file */
     while(fp) {
-	r = ReadLineFromFile();
+	r = ReadLineFromFile(use_pclose);
 	if (r) {
 	    DestroyCache(cf);
 	    ShouldCache = 0;
-	    FCLOSE(fp);
+            if (use_pclose) {
+                PCLOSE(fp);
+            } else {
+                FCLOSE(fp);
+            }
 	    return r;
 	}
 /* Skip blank chars */
@@ -406,7 +427,11 @@ static int CacheFile(char const *fname)
 		    DBufFree(&LineBuffer);
 		    DestroyCache(cf);
 		    ShouldCache = 0;
-		    FCLOSE(fp);
+                    if (use_pclose) {
+                        PCLOSE(fp);
+                    } else {
+                        FCLOSE(fp);
+                    }
 		    return E_NO_MEM;
 		}
 		cl = cf->cache;
@@ -416,7 +441,11 @@ static int CacheFile(char const *fname)
 		    DBufFree(&LineBuffer);
 		    DestroyCache(cf);
 		    ShouldCache = 0;
-		    FCLOSE(fp);
+                    if (use_pclose) {
+                        PCLOSE(fp);
+                    } else {
+                        FCLOSE(fp);
+                    }
 		    return E_NO_MEM;
 		}
 		cl = cl->next;
@@ -428,7 +457,11 @@ static int CacheFile(char const *fname)
 	    if (!cl->text) {
 		DestroyCache(cf);
 		ShouldCache = 0;
-		FCLOSE(fp);
+                if (use_pclose) {
+                    PCLOSE(fp);
+                } else {
+                    FCLOSE(fp);
+                }
 		return E_NO_MEM;
 	    }
 	}
@@ -471,9 +504,6 @@ static int PopFile(void)
 {
     IncludeStruct *i;
 
-    /* Assume we own the file for now */
-    RunDisabled &= ~RUN_NOTOWNER;
-
     if (!Hush && NumIfs) Eprint("%s", ErrMsg[E_MISS_ENDIF]);
     if (!IStackPtr) return E_EOF;
     i = &IStack[IStackPtr-1];
@@ -500,8 +530,10 @@ static int PopFile(void)
     STRSET(FileName, i->filename);
     if (!i->ownedByMe) {
 	RunDisabled |= RUN_NOTOWNER;
+    } else {
+	RunDisabled &= ~RUN_NOTOWNER;
     }
-    if (!CLine && (i->offset != -1L)) {
+    if (!CLine && (i->offset != -1L || !strcmp(i->filename, "-"))) {
 	/* We must open the file, then seek to specified position */
 	if (strcmp(i->filename, "-")) {
 	    fp = fopen(i->filename, "r");
@@ -544,6 +576,64 @@ int DoInclude(ParsePtr p)
     return OK;
 }
 
+/***************************************************************/
+/*                                                             */
+/*  DoIncludeCmd                                               */
+/*                                                             */
+/*  The INCLUDECMD command.                                    */
+/*                                                             */
+/***************************************************************/
+int DoIncludeCmd(ParsePtr p)
+{
+    DynamicBuffer buf;
+    int r;
+    int ch;
+    char append_buf[2];
+    int seen_nonspace = 0;
+
+    append_buf[1] = 0;
+
+    DBufInit(&buf);
+
+    while(1) {
+        ch = ParseChar(p, &r, 0);
+        if (r) {
+	    DBufFree(&buf);
+	    return r;
+	}
+        if (!ch) {
+            break;
+        }
+        if (isspace(ch) && !seen_nonspace) {
+            continue;
+        }
+        seen_nonspace = 1;
+        /* Convert \n to ' ' to better handle line continuation */
+        if (ch == '\n') {
+            ch = ' ';
+        }
+        append_buf[0] = (char) ch;
+	if (DBufPuts(&buf, append_buf) != OK) {
+	    DBufFree(&buf);
+	    return E_NO_MEM;
+	}
+    }
+
+    if (RunDisabled) {
+        DBufFree(&buf);
+        return E_RUN_DISABLED;
+    }
+
+    if ( (r=IncludeCmd(DBufValue(&buf))) ) {
+	DBufFree(&buf);
+	return r;
+    }
+    DBufFree(&buf);
+    NumIfs = 0;
+    IfFlags = 0;
+    return OK;
+}
+
 #ifdef HAVE_GLOB
 static int SetupGlobChain(char const *dirname, IncludeStruct *i)
 {
@@ -664,6 +754,127 @@ static int SetupGlobChain(char const *dirname, IncludeStruct *i)
 }
 #endif
 
+/***************************************************************/
+/*                                                             */
+/*  IncludeCmd                                                 */
+/*                                                             */
+/*  Process the INCLUDECMD command - actually do the command   */
+/*  inclusion.                                                 */
+/*                                                             */
+/***************************************************************/
+static int IncludeCmd(char const *cmd)
+{
+    IncludeStruct *i;
+    DynamicBuffer buf;
+    FILE *fp2;
+    int r;
+    CachedFile *h;
+    char const *fname;
+    int old_flag;
+
+    FreshLine = 1;
+    if (IStackPtr+1 >= INCLUDE_NEST) return E_NESTED_INCLUDE;
+    i = &IStack[IStackPtr];
+
+    /* Use "cmd|" as the filename */
+    DBufInit(&buf);
+    if (DBufPuts(&buf, cmd) != OK) {
+	DBufFree(&buf);
+	return E_NO_MEM;
+    }
+    if (DBufPuts(&buf, "|") != OK) {
+	DBufFree(&buf);
+	return E_NO_MEM;
+    }
+    fname = DBufValue(&buf);
+
+    if (FileName) {
+	i->filename = StrDup(FileName);
+	if (!i->filename) {
+	    DBufFree(&buf);
+	    return E_NO_MEM;
+	}
+    } else {
+	i->filename = NULL;
+    }
+    i->ownedByMe = 1;
+    i->LineNo = LineNo;
+    i->NumIfs = NumIfs;
+    i->IfFlags = IfFlags;
+    i->CLine = CLine;
+    i->offset = -1L;
+    i->chain = NULL;
+    if (fp) {
+	i->offset = ftell(fp);
+	FCLOSE(fp);
+    }
+    IStackPtr++;
+
+    /* If the file is cached, use it */
+    h = CachedFiles;
+    while(h) {
+        if (!strcmp(fname, h->filename)) {
+            if (DebugFlag & DB_TRACE_FILES) {
+                fprintf(ErrFp, "Reading command `%s': Found in cache\n", fname);
+            }
+            CLine = h->cache;
+            STRSET(FileName, fname);
+            DBufFree(&buf);
+            LineNo = 0;
+            if (!h->ownedByMe) {
+                RunDisabled |= RUN_NOTOWNER;
+            } else {
+                RunDisabled &= ~RUN_NOTOWNER;
+            }
+            if (FileName) return OK; else return E_NO_MEM;
+        }
+        h = h->next;
+    }
+
+    if (DebugFlag & DB_TRACE_FILES) {
+        fprintf(ErrFp, "Executing `%s' for INCLUDECMD and caching as `%s'\n",
+                cmd, fname);
+    }
+
+    /* Not found in cache */
+
+    /* If cmd starts with !, then disable RUN within the cmd output */
+    if (cmd[0] == '!') {
+        fp2 = popen(cmd+1, "r");
+    } else {
+        fp2 = popen(cmd, "r");
+    }
+    if (!fp2) {
+	DBufFree(&buf);
+	return E_CANT_OPEN;
+    }
+    fp = fp2;
+    LineNo = 0;
+
+    /* Temporarily turn of file tracing */
+    old_flag = DebugFlag;
+    DebugFlag &= (~DB_TRACE_FILES);
+
+    if (cmd[0] == '!') {
+        RunDisabled |= RUN_NOTOWNER;
+    }
+    r = CacheFile(fname, 1);
+
+    DebugFlag = old_flag;
+    if (r == OK) {
+	fp = NULL;
+	CLine = CachedFiles->cache;
+	LineNo = 0;
+	STRSET(FileName, fname);
+	DBufFree(&buf);
+	return OK;
+    }
+    DBufFree(&buf);
+    /* We failed */
+    PopFile();
+    return E_CANT_OPEN;
+}
+
 /***************************************************************/
 /*                                                             */
 /*  IncludeFile                                                */
@@ -869,6 +1080,8 @@ static int CheckSafety(void)
 /* If file is not owned by me, disable RUN command */
     if (statbuf.st_uid != geteuid()) {
 	RunDisabled |= RUN_NOTOWNER;
+    } else {
+	RunDisabled &= ~RUN_NOTOWNER;
     }
 
     return 1;
