@@ -5,7 +5,7 @@
 /*  Function Prototypes.                                       */
 /*                                                             */
 /*  This file is part of REMIND.                               */
-/*  Copyright (C) 1992-2020 by Dianne Skoll                    */
+/*  Copyright (C) 1992-2021 by Dianne Skoll                    */
 /*                                                             */
 /***************************************************************/
 
@@ -45,6 +45,7 @@ int CopyValue (Value *dest, const Value *src);
 int ReadLine (void);
 int OpenFile (char const *fname);
 int DoInclude (ParsePtr p);
+int DoIncludeCmd (ParsePtr p);
 int IncludeFile (char const *fname);
 int GetAccessDate (char const *file);
 int SetAccessDate (char const *fname, int jul);
@@ -161,3 +162,4 @@ void PrintJSONKeyPairDate(char const *name, int jul);
 void PrintJSONKeyPairDateTime(char const *name, int dt);
 void PrintJSONKeyPairTime(char const *name, int t);
 void System(char const *cmd);
+int ShellEscape(char const *in, DynamicBuffer *out);
